class PyHPKEError(Exception):
    """
    Base class for all exceptions.
    """

    pass


class NotSupportedError(PyHPKEError):
    """
    An Exception occurred when the function is not supported.
    """

    pass


class SealError(PyHPKEError):
    """
    An Exception occurred when an encryption process failed.
    """

    pass


class OpenError(PyHPKEError):
    """
    An Exception occurred when an decryption process failed.
    """

    pass



from .aead_key_interface import AEADKeyInterface
from .consts import AEADId


class AEADInterface:
    """
    The AEAD (Authenticated Encapsulation with Additional Data) interface.
    """

    @property
    def id(self) -> AEADId:
        """
        The AEAD identifier.
        """
        raise NotImplementedError()

    @property
    def key_size(self) -> int:
        """
        The AEAD key size.
        """
        raise NotImplementedError()

    @property
    def nonce_size(self) -> int:
        """
        The AEAD nonce size.
        """
        raise NotImplementedError()

    @property
    def tag_size(self) -> int:
        """
        The AEAD tag size.
        """
        raise NotImplementedError()

    def import_key(self, key: bytes) -> AEADKeyInterface:
        """
        Imports a byte string as an AEAD key.
        """
        raise NotImplementedError()



from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import ec

from ..consts import KEMId
from ..kem_key import KEMKeyPair
from ..kem_key_interface import KEMKeyInterface
from ..kem_primitives_interface import KEMPrimitivesInterface
from ..keys.ec_key import ECKey

# from cryptography.hazmat.primitives.serialization import Encoding, PublicFormat


class EC(KEMPrimitivesInterface):
    """
    The KEM (Key Encapsulation Mechanism) context.
    """

    def __init__(self, kem_id: KEMId):
        if kem_id == KEMId.DHKEM_P256_HKDF_SHA256:
            self._crv = ec.SECP256R1()
            self._nsecret = 32
        elif kem_id == KEMId.DHKEM_P384_HKDF_SHA384:
            self._crv = ec.SECP384R1()
            self._nsecret = 48
        elif kem_id == KEMId.DHKEM_P521_HKDF_SHA512:
            self._crv = ec.SECP521R1()
            self._nsecret = 64
        else:
            raise ValueError(f"Invalid kem_id: {kem_id}")

    def generate_key_pair(self) -> KEMKeyPair:
        sk = ec.generate_private_key(self._crv, backend=default_backend())
        pk = sk.public_key()
        return KEMKeyPair(ECKey(sk), ECKey(pk))

    def derive_key_pair(self, ikm: bytes) -> KEMKeyPair:
        raise NotImplementedError()

    def deserialize_private_key(self, key: bytes) -> KEMKeyInterface:
        return ECKey.from_private_bytes(self._crv, key)

    def serialize_public_key(self, key: KEMKeyInterface) -> bytes:
        return key.to_public_bytes()

    def deserialize_public_key(self, key: bytes) -> KEMKeyInterface:
        return ECKey.from_public_bytes(self._crv, key)

    def derive_public_key(self, sk: KEMKeyInterface) -> KEMKeyInterface:
        return ECKey(sk.raw.public_key())

    def exchange(self, sk: KEMKeyInterface, pk: KEMKeyInterface) -> bytes:
        return sk.raw.exchange(ec.ECDH(), pk.raw)



from .aead_key_interface import AEADKeyInterface
from .consts import AEADId
from .keys.aes_gcm_key import AESGCMKey
from .keys.chacha20_poly1305_key import ChaCha20Poly1305Key


class AEADKey:
    """
    A :class:`AEADKeyInterface <hpke.AEADKeyInterface>` Builder.
    """

    @classmethod
    def from_bytes(cls, aead_id: AEADId, data: bytes) -> AEADKeyInterface:
        """
        Creates an AEAD key from a byte string.
        """
        if aead_id == AEADId.AES128_GCM:
            return AESGCMKey(data, 16)
        if aead_id == AEADId.AES256_GCM:
            return AESGCMKey(data, 32)
        if aead_id == AEADId.CHACHA20_POLY1305:
            return ChaCha20Poly1305Key(data)
        raise ValueError(f"Unsupported or unknown AEAD id: {aead_id}.")



from pyhpke import AEADId, CipherSuite, KDFId, KEMId, KEMKey


class TestSample:
    """
    Tests for sample code.
    """

    def test_hpke_ec(self):

        # The sender side:
        suite_s = CipherSuite.new(KEMId.DHKEM_P256_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES128_GCM)
        pkr = KEMKey.from_jwk(
            {
                "kid": "01",
                "kty": "EC",
                "crv": "P-256",
                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
            }
        )
        enc, sender = suite_s.create_sender_context(pkr)
        ct = sender.seal(b"Hello world!")

        # The recipient side:
        suite_r = CipherSuite.new(KEMId.DHKEM_P256_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES128_GCM)
        skr = KEMKey.from_jwk(
            {
                "kid": "01",
                "kty": "EC",
                "crv": "P-256",
                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
                "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
            }
        )
        recipient = suite_r.create_recipient_context(enc, skr)
        pt = recipient.open(ct)

        assert pt == b"Hello world!"

    def test_hpke_x25519(self):

        # The sender side:
        suite_s = CipherSuite.new(KEMId.DHKEM_X25519_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES256_GCM)
        pkr = KEMKey.from_jwk(
            {
                "kid": "X25519-01",
                "kty": "OKP",
                "crv": "X25519",
                "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
            }
        )
        enc, sender = suite_s.create_sender_context(pkr)
        ct = sender.seal(b"Hello world!")

        # The recipient side:
        suite_r = CipherSuite.new(KEMId.DHKEM_X25519_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES256_GCM)
        skr = KEMKey.from_jwk(
            {
                "kid": "X25519-01",
                "kty": "OKP",
                "crv": "X25519",
                "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
                "d": "vsJ1oX5NNi0IGdwGldiac75r-Utmq3Jq4LGv48Q_Qc4",
            }
        )
        recipient = suite_r.create_recipient_context(enc, skr)
        pt = recipient.open(ct)

        assert pt == b"Hello world!"

    def test_hpke_x25519_from_pem(self):

        public_key_pem = b"-----BEGIN PUBLIC KEY-----\nMCowBQYDK2VuAyEAoMfvlI5DN08JRFP2fhWvZ6vBEl28yFeS9O9YQUjNyCY=\n-----END PUBLIC KEY-----"
        pkr = KEMKey.from_pem(public_key_pem)
        suite = CipherSuite.new(KEMId.DHKEM_X25519_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES128_GCM)
        enc, sender = suite.create_sender_context(pkr)
        ct = sender.seal(b"Hello world!")

        private_key_pem = b"-----BEGIN PRIVATE KEY-----\nMC4CAQAwBQYDK2VuBCIEIMAXvyHjAeXy9x4MXF6rwGbDKw7crgDriFTFXO+XsS1F\n-----END PRIVATE KEY-----"
        skr = KEMKey.from_pem(private_key_pem)
        recipient = suite.create_recipient_context(enc, skr)
        pt = recipient.open(ct)

        assert pt == b"Hello world!"

    def test_hpke_x448(self):

        # The sender side:
        suite_s = CipherSuite.new(KEMId.DHKEM_X448_HKDF_SHA512, KDFId.HKDF_SHA512, AEADId.CHACHA20_POLY1305)
        pkr = KEMKey.from_jwk(
            {
                "kid": "X448-01",
                "kty": "OKP",
                "crv": "X448",
                "x": "IkLmc0klvEMXYneHMKAB6ePohryAwAPVe2pRSffIDY6NrjeYNWVX5J-fG4NV2OoU77C88A0mvxI",
            }
        )
        enc, sender = suite_s.create_sender_context(pkr)
        ct = sender.seal(b"Hello world!")

        # The recipient side:
        suite_r = CipherSuite.new(KEMId.DHKEM_X448_HKDF_SHA512, KDFId.HKDF_SHA512, AEADId.CHACHA20_POLY1305)
        skr = KEMKey.from_jwk(
            {
                "kid": "X448-01",
                "kty": "OKP",
                "crv": "X448",
                "x": "IkLmc0klvEMXYneHMKAB6ePohryAwAPVe2pRSffIDY6NrjeYNWVX5J-fG4NV2OoU77C88A0mvxI",
                "d": "rJJRG3nshyCtd9CgXld8aNaB9YXKR0UOi7zj7hApg9YH4XdBO0G8NcAFNz_uPH2GnCZVcSDgV5c",
            }
        )
        recipient = suite_r.create_recipient_context(enc, skr)
        pt = recipient.open(ct)

        assert pt == b"Hello world!"



from cryptography.hazmat.primitives.asymmetric.x448 import X448PrivateKey

from ..kem_key import KEMKeyPair
from ..kem_key_interface import KEMKeyInterface
from ..kem_primitives_interface import KEMPrimitivesInterface
from ..keys.x448_key import X448Key


class X448(KEMPrimitivesInterface):
    """ """

    def __init__(self):
        self._nsecret = 64

    def generate_key_pair(self) -> KEMKeyPair:
        sk = X448PrivateKey.generate()
        pk = sk.public_key()
        return KEMKeyPair(X448Key(sk), X448Key(pk))

    def derive_key_pair(self, ikm: bytes) -> KEMKeyPair:
        raise NotImplementedError()

    def deserialize_private_key(self, key: bytes) -> KEMKeyInterface:
        return X448Key.from_private_bytes(key)

    def serialize_public_key(self, pk: KEMKeyInterface) -> bytes:
        return pk.to_public_bytes()

    def deserialize_public_key(self, pk: bytes) -> KEMKeyInterface:
        return X448Key.from_public_bytes(pk)

    def derive_public_key(self, sk: KEMKeyInterface) -> KEMKeyInterface:
        return X448Key(sk.raw.public_key())

    def exchange(self, sk: KEMKeyInterface, pk: KEMKeyInterface) -> bytes:
        return sk.raw.exchange(pk.raw)



import base64


def xor_bytes(b1: bytes, b2: bytes) -> bytes:
    return bytes([a1 ^ a2 for (a1, a2) in zip(b1, b2)])


def base64url_decode(v: str) -> bytes:
    bv = v.encode("ascii")
    rem = len(bv) % 4
    if rem > 0:
        bv += b"=" * (4 - rem)
    return base64.urlsafe_b64decode(bv)


def base64url_encode(input: bytes) -> bytes:
    return base64.urlsafe_b64encode(input).replace(b"=", b"")



import enum

HPKE_VERSION = b"HPKE-v1"


class Mode(enum.Enum):
    BASE = 0
    PSK = 1
    AUTH = 2
    AUTH_PSK = 3


class KEMId(enum.Enum):
    DHKEM_P256_HKDF_SHA256 = 0x0010
    DHKEM_P384_HKDF_SHA384 = 0x0011
    DHKEM_P521_HKDF_SHA512 = 0x0012
    DHKEM_X25519_HKDF_SHA256 = 0x0020
    DHKEM_X448_HKDF_SHA512 = 0x0021


class KDFId(enum.Enum):
    HKDF_SHA256 = 0x0001
    HKDF_SHA384 = 0x0002
    HKDF_SHA512 = 0x0003


class AEADId(enum.Enum):
    AES128_GCM = 0x0001
    AES256_GCM = 0x0002
    CHACHA20_POLY1305 = 0x0003
    EXPORT_ONLY = 0xFFFF


HPKE_SUPPORTED_JWK_KTYS = ["EC", "OKP"]
HPKE_SUPPORTED_JWK_EC_CRVS = ["P-256", "P-384", "P-521"]



from .encryption_context import EncryptionContext
from .exceptions import NotSupportedError, OpenError


class RecipientContext(EncryptionContext):
    def seal(self, pt: bytes, aad: bytes = b"") -> bytes:
        raise NotSupportedError("Not available for recipient.")

    def open(self, ct: bytes, aad: bytes = b"") -> bytes:
        try:
            return self._key.open(ct, self._next_nonce(), aad)
        except Exception as err:
            raise OpenError("Failed to open.") from err



from typing import Optional, Tuple

from .consts import KEMId
from .kem_key import KEMKeyPair
from .kem_key_interface import KEMKeyInterface


class KEMInterface(object):
    """
    The KEM (Key Encapsulation Mechanism) interface.
    """

    @property
    def id(self) -> KEMId:
        """
        The KEM identifier.
        """
        raise NotImplementedError()

    def deserialize_private_key(self, key: bytes) -> KEMKeyInterface:
        raise NotImplementedError()

    def deserialize_public_key(self, key: bytes) -> KEMKeyInterface:
        raise NotImplementedError()

    def encap(
        self, pkr: KEMKeyInterface, sks: Optional[KEMKeyInterface] = None, eks: Optional[KEMKeyPair] = None
    ) -> Tuple[bytes, bytes]:
        raise NotImplementedError()

    def decap(self, enc: bytes, skr: KEMKeyInterface, pks: Optional[KEMKeyInterface] = None) -> bytes:
        raise NotImplementedError()



from typing import Any


class KEMKeyInterface:
    """
    The KEM key interface.
    """

    def __init__(self, key: Any):
        self._key = key

    @property
    def raw(self) -> Any:
        return self._key

    def to_public_bytes(self) -> bytes:
        """
        Serializes the key to a byte string if it is public.
        """
        raise NotImplementedError()



from .consts import KDFId


class KDFInterface(object):
    """
    The KDF (Key Derivation Function) interface.
    """

    @property
    def id(self) -> KDFId:
        """
        The KDF identifier.
        """
        raise NotImplementedError()

    def extract(self, salt: bytes, ikm: bytes) -> bytes:
        raise NotImplementedError()

    def expand(self, prk: bytes, info: bytes, length: int) -> bytes:
        raise NotImplementedError()

    # def extract_and_expand(self, salt: bytes, ikm: bytes, info: bytes, length: int) -> bytes:
    #     raise NotImplementedError()

    # def labeled_extract(self, salt: bytes, label: bytes, ikm: bytes) -> bytes:
    #     raise NotImplementedError()
    #
    # def labeled_expand(self, prk: bytes, label: bytes, info: bytes, length: int) -> bytes:
    #     raise NotImplementedError()



from cryptography.hazmat.primitives.ciphers import aead

from ..aead_key_interface import AEADKeyInterface


class ChaCha20Poly1305Key(AEADKeyInterface):
    """
    The ChaCha20-Poly1305 key.
    """

    def __init__(self, key: bytes):
        if len(key) != 32:
            raise ValueError(f"Invalid key size: {len(key)}.")
        self._ctx = aead.ChaCha20Poly1305(key)

    def seal(self, pt: bytes, nonce: bytes, aad: bytes = b"") -> bytes:
        return self._ctx.encrypt(nonce, pt, aad)

    def open(self, ct: bytes, nonce: bytes, aad: bytes = b"") -> bytes:
        return self._ctx.decrypt(nonce, ct, aad)



from .kem_key import KEMKeyInterface, KEMKeyPair


class KEMPrimitivesInterface(object):
    """
    The KEM (Key Encapsulation Mechanism) interface.
    """

    def generate_key_pair(self) -> KEMKeyPair:
        raise NotImplementedError()

    def derive_key_pair(self, ikm: bytes) -> KEMKeyPair:
        raise NotImplementedError()

    def serialize_private_key(self, key: bytes) -> KEMKeyInterface:
        raise NotImplementedError()

    def serialize_public_key(self, key: KEMKeyInterface) -> bytes:
        raise NotImplementedError()

    def deserialize_private_key(self, key: bytes) -> KEMKeyInterface:
        raise NotImplementedError()

    def deserialize_public_key(self, key: bytes) -> KEMKeyInterface:
        raise NotImplementedError()

    def derive_public_key(self, sk: KEMKeyInterface) -> KEMKeyInterface:
        raise NotImplementedError()

    def exchange(self, sk: KEMKeyInterface, pk: KEMKeyInterface) -> bytes:
        raise NotImplementedError()

    # def encap(self, pkr: KEMKeyInterface, sks: Optional[KEMKeyInterface] = None) -> Tuple[bytes, bytes]:
    #     raise NotImplementedError()
    #
    # def decap(self, enc: bytes, skr: KEMKeyInterface, pks: Optional[KEMKeyInterface]) -> bytes:
    #     raise NotImplementedError()
    #



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyhpke', 'pyhpke.kem_primitives', 'pyhpke.keys']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=36,<39']

extras_require = \
{'docs': ['Sphinx[docs]>=4.3.2,<6.0.0',
          'sphinx-autodoc-typehints[docs]==1.19.5',
          'sphinx-rtd-theme[docs]>=1.0.0,<2.0.0']}

setup_kwargs = {
    'name': 'pyhpke',
    'version': '0.3.0',
    'description': 'A Python implementation of HPKE.',
    'long_description': '# PyHPKE - A Python implementation of HPKE\n\n[![PyPI version](https://badge.fury.io/py/pyhpke.svg)](https://badge.fury.io/py/pyhpke)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/pyhpke)\n[![Documentation Status](https://readthedocs.org/projects/pyhpke/badge/?version=latest)](https://pyhpke.readthedocs.io/en/latest/?badge=latest)\n![Github CI](https://github.com/dajiaji/pyhpke/actions/workflows/python-package.yml/badge.svg)\n[![codecov](https://codecov.io/gh/dajiaji/pyhpke/branch/main/graph/badge.svg?token=QN8GXEYEP3)](https://codecov.io/gh/dajiaji/pyhpke)\n\n\nPyHPKE is a [HPKE (Hybrid Public Key Encryption)](https://www.rfc-editor.org/rfc/rfc9180.html) implementation written in Python.\n\nYou can install PyHPKE with pip:\n\n```sh\n$ pip install pyhpke\n```\n\nAnd then, you can use it as follows:\n\n\n```py\nfrom pyhpke import AEADId, CipherSuite, KDFId, KEMId, KEMKey\n\n# The sender side:\nsuite_s = CipherSuite.new(KEMId.DHKEM_P256_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES128_GCM)\npkr = KEMKey.from_jwk(  # from_pem is also available.\n    {\n        "kid": "01",\n        "kty": "EC",\n        "crv": "P-256",\n        "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",\n        "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",\n    }\n)\nenc, sender = suite_s.create_sender_context(pkr)\nct = sender.seal(b"Hello world!")\n\n# The recipient side:\nsuite_r = CipherSuite.new(KEMId.DHKEM_P256_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES128_GCM)\nskr = KEMKey.from_jwk(\n    {\n        "kid": "01",\n        "kty": "EC",\n        "crv": "P-256",\n        "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",\n        "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",\n        "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",\n    }\n)\nrecipient = suite_r.create_recipient_context(enc, skr)\npt = recipient.open(ct)\n\nassert pt == b"Hello world!"\n```\n\n## Index\n\n- [Installation](#installation)\n- [Supported HPKE Modes and Cipher Suites](#supported-hpke-modes-and-cipher-suites)\n- [Usage](#usage)\n- [API Reference](#api-reference)\n- [Test](#test)\n- [Contributing](#contributing)\n\n## Installation\n\nYou can install PySETO with pip:\n\n```sh\n$ pip install pyseto\n```\n\n## Supported HPKE Modes and Cipher Suites\n\n\n### Modes\n\n| Base | PSK | Auth | AuthPSK |\n| ---- | --- | ---- | ------- |\n| ✅   | ✅  | ✅   | ✅      |\n\n\n### Key Encapsulation Machanisms (KEMs)\n\n| KEMs                        |    |\n| --------------------------- | -- |\n| DHKEM (P-256, HKDF-SHA256)  | ✅ |\n| DHKEM (P-384, HKDF-SHA384)  | ✅ |\n| DHKEM (P-521, HKDF-SHA512)  | ✅ |\n| DHKEM (X25519, HKDF-SHA256) | ✅ |\n| DHKEM (X448, HKDF-SHA512)   | ✅ |\n\n\n### Key Derivation Functions (KDFs)\n\n| KDFs        |    |\n| ----------- | -- |\n| HKDF-SHA256 | ✅ |\n| HKDF-SHA384 | ✅ |\n| HKDF-SHA512 | ✅ |\n\n\n### Authenticated Encryption with Associated Data (AEAD) Functions\n\n| AEADs            |    |\n| ---------------- | -- |\n| AES-128-GCM      | ✅ |\n| AES-256-GCM      | ✅ |\n| ChaCha20Poly1305 | ✅ |\n| Export Only      | ✅ |\n\n## Usage\n\n```py\nfrom pyhpke import AEADId, CipherSuite, KDFId, KEMId, KEMKey\n\n# The sender side:\nsuite_s = CipherSuite.new(KEMId.DHKEM_P256_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES128_GCM)\npkr = KEMKey.from_jwk(\n    {\n        "kid": "01",\n        "kty": "EC",\n        "crv": "P-256",\n        "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",\n        "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",\n    }\n)\nenc, sender = suite_s.create_sender_context(pkr)\nct = sender.seal(b"Hello world!")\n\n# The recipient side:\nsuite_r = CipherSuite.new(KEMId.DHKEM_P256_HKDF_SHA256, KDFId.HKDF_SHA256, AEADId.AES128_GCM)\nskr = KEMKey.from_jwk(\n    {\n        "kid": "01",\n        "kty": "EC",\n        "crv": "P-256",\n        "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",\n        "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",\n        "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",\n    }\n)\nrecipient = suite_r.create_recipient_context(enc, skr)\npt = recipient.open(ct)\n\nassert pt == b"Hello world!"\n```\n\n## API Reference\n\nSee [Documentation](https://pyhpke.readthedocs.io/en/stable/api.html).\n\n## Test\n\nYou can run tests from the project root after cloning with:\n\n```sh\n$ tox\n```\n\n## Contributing\n\nWe welcome all kind of contributions, filing issues, suggesting new features or sending PRs.\n',
    'author': 'Ajitomi Daisuke',
    'author_email': 'dajiaji@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dajiaji/pyhpke',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7.0,<4.0.0',
}


setup(**setup_kwargs)



from .aead_interface import AEADInterface
from .aead_key import AEADKey
from .aead_key_interface import AEADKeyInterface
from .consts import AEADId


class AEAD(AEADInterface):
    """
    An AEAD(Authenticated Encapsulation with Additional Data) context.
    """

    def __init__(self, aead_id: AEADId):
        """
        Creates an AEAD context.

        Args:
            aead_id (int): An AEAD (Authenticated Encryption with Additional Data) identifier.
        Returns:
            bytes: An AEAD context object.
        """
        if aead_id == AEADId.AES128_GCM:
            self._nk = 16
            self._nn = 12
            self._nt = 16
        elif aead_id == AEADId.AES256_GCM:
            self._nk = 32
            self._nn = 12
            self._nt = 16
        elif aead_id == AEADId.CHACHA20_POLY1305:
            self._nk = 32
            self._nn = 12
            self._nt = 16
        elif aead_id == AEADId.EXPORT_ONLY:
            self._nk = 0
            self._nn = 0
            self._nt = 0
        else:
            raise ValueError(f"Invalid aead_id: {aead_id}.")
        self._id = aead_id
        return

    @property
    def id(self) -> AEADId:
        """
        The AEAD identifier.
        """
        return self._id

    @property
    def key_size(self) -> int:
        """
        The AEAD key size.
        """
        return self._nk

    @property
    def nonce_size(self) -> int:
        """
        The AEAD nonce size.
        """
        return self._nn

    @property
    def tag_size(self) -> int:
        """
        The AEAD tag size.
        """
        return self._nt

    def import_key(self, key: bytes) -> AEADKeyInterface:
        """
        Imports a byte string as an AEAD key.
        """
        return AEADKey.from_bytes(self._id, key)


class AEADParams(object):
    def __init__(self, ctx: AEADInterface, key: bytes, base_nonce: bytes, seq: int, exporter_secret: bytes):
        self._ctx = ctx
        self._key = key
        self._base_nonce = base_nonce
        self._seq = seq
        self._exporter_secret = exporter_secret

    @property
    def ctx(self) -> AEADInterface:
        return self._ctx

    @property
    def key(self) -> bytes:
        return self._key

    @property
    def base_nonce(self) -> bytes:
        return self._base_nonce

    @property
    def seq(self) -> int:
        return self._seq

    @property
    def exporter_secret(self) -> bytes:
        return self._exporter_secret



from .aead_interface import AEADInterface
from .aead_key_interface import AEADKeyInterface
from .cipher_suite import CipherSuite
from .consts import AEADId, KDFId, KEMId
from .context_interface import ContextInterface
from .exceptions import NotSupportedError, OpenError, PyHPKEError, SealError
from .kdf_interface import KDFInterface
from .kem_interface import KEMInterface
from .kem_key import KEMKey, KEMKeyPair
from .kem_key_interface import KEMKeyInterface

__version__ = "0.3.0"
__title__ = "PyHPKE"
__description__ = "A Python implementation of HPKE."
__url__ = "https://pyhpke.readthedocs.io"
__uri__ = __url__
__doc__ = __description__ + " <" + __uri__ + ">"
__author__ = "AJITOMI Daisuke"
__email__ = "dajiaji@gmail.com"
__license__ = "MIT"
__copyright__ = "Copyright 2022 Ajitomi Daisuke"
__all__ = [
    "AEADInterface",
    "AEADKeyInterface",
    "CipherSuite",
    "AEADId",
    "KDFId",
    "KEMId",
    "ContextInterface",
    "NotSupportedError",
    "OpenError",
    "PyHPKEError",
    "SealError",
    "KDFInterface",
    "KEMInterface",
    "KEMKey",
    "KEMKeyPair",
    "KEMKeyInterface",
]
